#!/usr/bin/env ruby
# -*- encoding: binary -*-

# Example of using IO.splice to copy a file
# This can be significantly faster than IO.copy_stream as data
# is never copied into userspace.

require 'io/splice'

usage = "#$0 SOURCE DEST"
source = ARGV.shift or abort usage
dest = ARGV.shift or abort usage

source = File.open(source, 'rb')
dest = File.open(dest, 'wb')
source_fd, dest_fd = source.fileno, dest.fileno

# We use a pipe as a ring buffer in kernel space.
# pipes may store up to IO::Splice::PIPE_CAPA bytes
pipe = IO.pipe
rfd, wfd = pipe.map { |io| io.fileno }

begin
  nread = begin
    # first pull as many bytes as possible into the pipe
    IO.splice(source_fd, nil, wfd, nil, IO::Splice::PIPE_CAPA, 0)
  rescue EOFError
    break
  end

  # now move the contents of the pipe buffer into the destination file
  # the copied data never enters userspace
  nwritten = IO.splice(rfd, nil, dest_fd, nil, nread, 0)

  nwritten == nread or
    abort "short splice to destination file: #{nwritten} != #{nread}"
end while true
